SystemOrganization addCategory: #'Multilingual-Tests-TextConversion'!

TestCase subclass: #ByteTextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!ByteTextConverterTest methodsFor: 'testing' stamp: 'nice 7/26/2009 22:44'!
testConversionToFrom
	"Non regresson test for http://code.google.com/p/pharo/issues/detail?id=986"

	self assert: (('' convertToEncoding: 'mac-roman') convertFromEncoding: 'mac-roman') = ''! !

TestCase subclass: #MultiByteFileStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!MultiByteFileStreamTest methodsFor: 'support' stamp: 'HenrikSperreJohansen 2/9/2010 11:09'!
chunkString
	"A chunk larger than buffer (currently 2048), but not long enough to fill it completely again. Therefore, buffer should contain some elements at end that are not part of the real buffer"
	| smallChunk chunkString |
	chunkString := 
	String new: (MultiByteFileStream new ascii enableReadBuffering braceArray size * 1.3) floor.
	smallChunk := 'ASDFASDFASDFASDFasdfasdfasdfasdfQWERQWERQWERqwerqwerqwer!! !!'.
	1 to: chunkString size by: smallChunk size do: [:ix |
		chunkString replaceFrom: ix to: (ix + smallChunk size -1 min: chunkString size) with: smallChunk startingAt: 1].
	^chunkString! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'SergeStinckwich 3/15/2010 17:23'!
tearDown
   FileDirectory default deleteFileNamed:'foobug6933'.! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 2/9/2010 11:20'!
testBasicChunk
	 | internalStream chunkKey |
	internalStream := self chunkString readStream.
	chunkKey := OrderedCollection new.
	[internalStream atEnd] whileFalse: 
		[ | chunk |
		chunk := internalStream nextChunk.
		chunkKey add: {internalStream position. chunk size. chunk}].
	[self writeChunkToFile.
	FileStream readOnlyFileNamed: 'chunkTest.txt' do: [:fileStream |
		1 to: chunkKey size do: [:ix | |chunk|
			chunk := fileStream nextChunk.
			self assert: (chunkKey at: ix) first equals: fileStream position.
			self assert: (chunkKey at: ix) second equals: chunk size.
			self assert: (chunkKey at: ix) last equals: chunk.]].
	] ensure: [FileDirectory default deleteFileNamed: 'chunkTest.txt']! !

!MultiByteFileStreamTest methodsFor: 'testing' stamp: 'StephaneDucasse 2/25/2010 18:05'!
testBinaryUpTo
	"This is a non regression test for bug http://bugs.squeak.org/view.php?id=6933"
	"self run: #testBinaryUpTo"
	
	| foo fileName |
	
	fileName := 'foobug6933'. 
	MultiByteFileStream 
		forceNewFileNamed: fileName 
		do: [ :stream | stream binary.
				stream nextPutAll: #[1 2 3 4] ].

	foo := MultiByteFileStream oldFileNamed: fileName.
	[foo binary.
		self assert: (foo upTo: 3) = #[1 2] 
	] ensure: [foo close]. 
    
     FileDirectory default deleteFileNamed: fileName! !

!MultiByteFileStreamTest methodsFor: 'support' stamp: 'HenrikSperreJohansen 2/9/2010 11:10'!
writeChunkToFile
	FileStream forceNewFileNamed: 'chunkTest.txt' do: [:stream | stream nextPutAll: self chunkString]! !

TestCase subclass: #UTF16TextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!UTF16TextConverterTest methodsFor: 'helpers' stamp: 'HenrikSperreJohansen 8/9/2010 16:25'!
assert: originalText isTheSameAfterLittleEndianConversion: aBooleanOrNil

	"Test that encoding / decoding a string witll produce the original string for both byte orders"

	| converter bytes decodedText |

	converter := UTF16TextConverter new.
	aBooleanOrNil ifNotNil: [converter useLittleEndian: aBooleanOrNil].
	bytes := (originalText convertToWithConverter: converter).
	decodedText := bytes convertFromWithConverter: converter.
	
	self assert: originalText equals: decodedText.
	! !

!UTF16TextConverterTest methodsFor: 'helpers' stamp: 'HenrikSperreJohansen 8/9/2010 15:58'!
assertBOMIsWrittenWithOrder: anOrder andConversion: aBlock

	|converter stream  BOM  |
	
	converter := UTF16TextConverter new.
	converter useByteOrderMark: true.
	
	anOrder = #bigEndian ifTrue: [converter useLittleEndian: false].
	anOrder = #littleEndian ifTrue: [converter useLittleEndian: true].
	
	BOM := (converter useLittleEndian 
				ifTrue: [16rFFFE]
				ifFalse: [16rFEFF]) asByteArray asString.
	
	stream := String new writeStream. 
	aBlock value: converter value: stream.
	
	self assert: BOM equals: (stream contents  first: 2).
	! !

!UTF16TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/9/2010 16:19'!
testBOMWrite
	
	"Test that BOM is written correctly for both endians, both when using a nextPut:, and nextPutAll:"
	
	| batchConversion oneByOneConversion |
	batchConversion := [:converter :stream | converter nextPutAll: 'test' toStream: stream].
	oneByOneConversion := [:converter :stream | 'test' do: [:each | converter nextPut: each toStream: stream]].
	
	self assertBOMIsWrittenWithOrder: #bigEndian 
		andConversion: oneByOneConversion.
	self assertBOMIsWrittenWithOrder: #bigEndian 
		andConversion: batchConversion.
		
	self assertBOMIsWrittenWithOrder: #littleEndian 
		andConversion: oneByOneConversion.
	self assertBOMIsWrittenWithOrder: #littleEndian 
		andConversion: batchConversion.
		
	self assertBOMIsWrittenWithOrder: nil 
		andConversion: oneByOneConversion.
	self assertBOMIsWrittenWithOrder: nil 
		andConversion: batchConversion.! !

!UTF16TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/9/2010 16:26'!
testByteOrders

	"Test that encoding / decoding a string witll produce the original string for both byte orders"

	"Default endianness"
	self assert: 'test' isTheSameAfterLittleEndianConversion: nil.
	"Big endian"
	self assert: 'test' isTheSameAfterLittleEndianConversion: false.
	"Little endian"
	self assert: 'test' isTheSameAfterLittleEndianConversion: true.
! !

!UTF16TextConverterTest methodsFor: 'testing' stamp: 'HenrikSperreJohansen 8/9/2010 16:43'!
testNoBOMWriteInMiddleOfFile
	
	"Test that BOM is not written when our first write does not occur at start of writestream"
	
	| converter stream |
	converter := UTF16TextConverter new.

	
	stream := String new writeStream.
	converter nextPutAll: 'testing' toStream: stream.
	stream position: 2.
	converter useByteOrderMark: true.
		
	converter nextPut: $a toStream: stream.
	self assert: 0 equals:  (stream contents indexOfSubCollection: 16rFEFF asByteArray asString)! !

TestCase subclass: #UTF8TextConverterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Multilingual-Tests-TextConversion'!

!UTF8TextConverterTest methodsFor: 'testing' stamp: 'pmm 5/22/2010 10:34'!
testPutSingleCharacter
	| actual |
	actual := ByteArray streamContents: [ :stream |
		| converter |
		converter := UTF8TextConverter new.
		converter
			nextPut: $a
			toStream: stream.
		converter
			nextPut: (Unicode value: 16r20AC)
			toStream: stream ].
	self assert: actual = #[97 0 0 32 172]! !
